@@ -486,6 +486,135 @@ void process_tags(snac *snac, const char *content, xs_str **n_content, xs_list *
 }
 
 
+void notify(snac *snac, const char *type, const char *utype, const char *actor, const xs_dict *msg)
+/* notifies the user of relevant events */
+{
+    const char *id = xs_dict_get(msg, "id");
+
+    if (strcmp(type, "Create") == 0) {
+        /* only notify of notes specifically for us */
+        xs *rcpts = recipient_list(snac, msg, 0);
+
+        if (xs_list_in(rcpts, snac->actor) == -1)
+            return;
+
+        /* discard votes */
+        const xs_dict *note = xs_dict_get(msg, "object");
+
+        if (note && !xs_is_null(xs_dict_get(note, "name")))
+            return;
+    }
+
+    if (strcmp(type, "Undo") == 0 && strcmp(utype, "Follow") != 0)
+        return;
+
+    /* get the object id */
+    const char *objid = xs_dict_get(msg, "object");
+
+    if (xs_type(objid) == XSTYPE_DICT)
+        objid = xs_dict_get(objid, "id");
+
+    if (strcmp(type, "Like") == 0 || strcmp(type, "Announce") == 0) {
+        /* if it's not an admiration about something by us, done */
+        if (xs_is_null(objid) || !xs_startswith(objid, snac->actor))
+            return;
+    }
+
+    /* updated poll? */
+    if (strcmp(type, "Update") == 0 && strcmp(utype, "Question") == 0) {
+        const xs_dict *poll;
+        const char *poll_id;
+
+        if ((poll = xs_dict_get(msg, "object")) == NULL)
+            return;
+
+        /* if it's not closed, discard */
+        if (xs_is_null(xs_dict_get(poll, "closed")))
+            return;
+
+        if ((poll_id = xs_dict_get(poll, "id")) == NULL)
+            return;
+
+        /* if it's not ours and we didn't vote, discard */
+        if (!xs_startswith(poll_id, snac->actor) && !was_question_voted(snac, poll_id))
+            return;
+    }
+
+    /* user will love to know about this! */
+
+    /* prepare message body */
+    xs *body = xs_fmt("User  : @%s@%s\n",
+        xs_dict_get(snac->config, "uid"),
+        xs_dict_get(srv_config,   "host")
+    );
+
+    if (strcmp(utype, "(null)") != 0) {
+        xs *s1 = xs_fmt("Type  : %s + %s\n", type, utype);
+        body = xs_str_cat(body, s1);
+    }
+    else {
+        xs *s1 = xs_fmt("Type  : %s\n", type);
+        body = xs_str_cat(body, s1);
+    }
+
+    {
+        xs *s1 = xs_fmt("Actor : %s\n", actor);
+        body = xs_str_cat(body, s1);
+    }
+
+    if (objid != NULL) {
+        xs *s1 = xs_fmt("Object: %s\n", objid);
+        body = xs_str_cat(body, s1);
+    }
+
+    /* email */
+
+    const char *email = "[disabled by admin]";
+
+    if (xs_type(xs_dict_get(srv_config, "disable_email_notifications")) != XSTYPE_TRUE) {
+        email = xs_dict_get(snac->config_o, "email");
+        if (xs_is_null(email)) {
+            email = xs_dict_get(snac->config, "email");
+
+            if (xs_is_null(email))
+                email = "[empty]";
+        }
+    }
+
+    if (*email != '\0' && *email != '[') {
+        snac_debug(snac, 1, xs_fmt("email notify %s %s %s", type, utype, actor));
+
+        xs *subject = xs_fmt("snac notify for @%s@%s",
+                    xs_dict_get(snac->config, "uid"), xs_dict_get(srv_config, "host"));
+        xs *from    = xs_fmt("snac-daemon <snac-daemon@%s>", xs_dict_get(srv_config, "host"));
+        xs *header  = xs_fmt(
+                    "From: %s\n"
+                    "To: %s\n"
+                    "Subject: %s\n"
+                    "\n",
+                    from, email, subject);
+
+        xs *email_body = xs_fmt("%s%s", header, body);
+
+        enqueue_email(email_body, 0);
+    }
+
+    /* telegram */
+
+    char *bot     = xs_dict_get(snac->config, "telegram_bot");
+    char *chat_id = xs_dict_get(snac->config, "telegram_chat_id");
+
+    if (!xs_is_null(bot) && !xs_is_null(chat_id) && *bot && *chat_id)
+        enqueue_telegram(body, bot, chat_id);
+
+    /* finally, store it in the notification folder */
+    if (strcmp(type, "Follow") == 0)
+        objid = id;
+
+    notify_add(snac, type, utype, actor, objid != NULL ? objid : id);
+}
+
+
 /** messages **/
 
 xs_dict *msg_base(snac *snac, const char *type, const char *id,
@@ -498,7 +627,7 @@ xs_dict *msg_base(snac *snac, const char *type, const char *id,
 
     /* generated values */
     if (date && strcmp(date, "@now") == 0) {
-        published = xs_str_utctime(0, "%Y-%m-%dT%H:%M:%SZ");
+        published = xs_str_utctime(0, ISO_DATE_SPEC);
         date = published;
     }
 
@@ -932,32 +1061,54 @@ xs_dict *msg_pong(snac *user, const char *rcpt, const char *object)
 }
 
 
-xs_dict *msg_question(snac *user, const char *content, const xs_list *opts, int multiple, int end_secs)
+xs_dict *msg_question(snac *user, const char *content, xs_list *attach,
+                      const xs_list *opts, int multiple, int end_secs)
 /* creates a Question message */
 {
     xs *ntid     = tid(0);
-    xs *id       = xs_fmt("%s/q/%s", user->actor, ntid);
-    xs_dict *msg = msg_base(user, "Question", id, NULL, "@now", NULL);
+    xs_dict *msg = msg_note(user, content, NULL, NULL, attach, 0);
+    int max      = 8;
+    xs_set seen;
 
-    msg = xs_dict_append(msg, "content",      content);
-    msg = xs_dict_append(msg, "attributedTo", user->actor);
+    msg = xs_dict_set(msg, "type", "Question");
+
+    /* make it non-editable */
+    msg = xs_dict_del(msg, "sourceContent");
 
     xs *o = xs_list_new();
     xs_list *p = (xs_list *)opts;
     xs_str *v;
+    xs *replies = xs_json_loads("{\"type\":\"Collection\",\"totalItems\":0}");
 
-    while (xs_list_iter(&p, &v)) {
-        xs *d = xs_dict_new();
+    xs_set_init(&seen);
 
-        d = xs_dict_append(d, "name", v);
-        o = xs_list_append(o, d);
+    while (max && xs_list_iter(&p, &v)) {
+        if (*v) {
+            xs *v2 = xs_dup(v);
+            xs *d  = xs_dict_new();
+
+            if (strlen(v2) > 60) {
+                v2[60] = '\0';
+                v2 = xs_str_cat(v2, "...");
+            }
+
+            if (xs_set_add(&seen, v2) == 1) {
+                d = xs_dict_append(d, "name",    v2);
+                d = xs_dict_append(d, "replies", replies);
+                o = xs_list_append(o, d);
+
+                max--;
+            }
+        }
     }
 
+    xs_set_free(&seen);
+
     msg = xs_dict_append(msg, multiple ? "anyOf" : "oneOf", o);
 
     /* set the end time */
     time_t t = time(NULL) + end_secs;
-    xs *et = xs_str_utctime(t, "%Y-%m-%dT%H:%M:%SZ");
+    xs *et = xs_str_utctime(t, ISO_DATE_SPEC);
 
     msg = xs_dict_append(msg, "endTime", et);
 
@@ -965,104 +1116,128 @@ xs_dict *msg_question(snac *user, const char *content, const xs_list *opts, int
 }
 
 
-void notify(snac *snac, xs_str *type, xs_str *utype, xs_str *actor, xs_dict *msg)
-/* notifies the user of relevant events */
+int update_question(snac *user, const char *id)
+/* updates the poll counts */
 {
-    if (strcmp(type, "Create") == 0) {
-        /* only notify of notes specifically for us */
-        xs *rcpts = recipient_list(snac, msg, 0);
+    xs *msg   = NULL;
+    xs *rcnt  = xs_dict_new();
+    xs *z     = xs_number_new(0);
+    xs *rcpts = xs_list_new();
+    xs *lopts = xs_list_new();
+    xs_list *opts;
+    xs_list *p;
+    xs_val *v;
 
-        if (xs_list_in(rcpts, snac->actor) == -1)
-            return;
-    }
+    /* get the object */
+    if (!valid_status(object_get(id, &msg)))
+        return -1;
 
-    if (strcmp(type, "Undo") == 0 && strcmp(utype, "Follow") != 0)
-        return;
+    /* closed? do nothing more */
+    if (xs_dict_get(msg, "closed"))
+        return -2;
 
-    /* get the object id */
-    const char *objid = xs_dict_get(msg, "object");
+    /* get the options */
+    if ((opts = xs_dict_get(msg, "oneOf")) == NULL &&
+        (opts = xs_dict_get(msg, "anyOf")) == NULL)
+        return -3;
 
-    if (xs_type(objid) == XSTYPE_DICT)
-        objid = xs_dict_get(objid, "id");
-
-    if (strcmp(type, "Like") == 0 || strcmp(type, "Announce") == 0) {
-        /* if it's not an admiration about something by us, done */
-        if (xs_is_null(objid) || !xs_startswith(objid, snac->actor))
-            return;
-    }
-
-    /* user will love to know about this! */
-
-    /* prepare message body */
-    xs *body = xs_fmt("User  : @%s@%s\n",
-        xs_dict_get(snac->config, "uid"),
-        xs_dict_get(srv_config,   "host")
-    );
-
-    if (strcmp(utype, "(null)") != 0) {
-        xs *s1 = xs_fmt("Type  : %s + %s\n", type, utype);
-        body = xs_str_cat(body, s1);
-    }
-    else {
-        xs *s1 = xs_fmt("Type  : %s\n", type);
-        body = xs_str_cat(body, s1);
-    }
-
-    {
-        xs *s1 = xs_fmt("Actor : %s\n", actor);
-        body = xs_str_cat(body, s1);
-    }
-
-    if (objid != NULL) {
-        xs *s1 = xs_fmt("Object: %s\n", objid);
-        body = xs_str_cat(body, s1);
-    }
-
-    /* email */
-
-    const char *email = "[disabled by admin]";
-
-    if (xs_type(xs_dict_get(srv_config, "disable_email_notifications")) != XSTYPE_TRUE) {
-        email = xs_dict_get(snac->config_o, "email");
-        if (xs_is_null(email)) {
-            email = xs_dict_get(snac->config, "email");
-
-            if (xs_is_null(email))
-                email = "[empty]";
+    /* fill the initial count */
+    p = opts;
+    while (xs_list_iter(&p, &v)) {
+        const char *name = xs_dict_get(v, "name");
+        if (name) {
+            lopts = xs_list_append(lopts, name);
+            rcnt  = xs_dict_set(rcnt, name, z);
         }
     }
 
-    if (*email != '\0' && *email != '[') {
-        snac_debug(snac, 1, xs_fmt("email notify %s %s %s", type, utype, actor));
+    /* iterate now the children (the votes) */
+    xs *chld = object_children(id);
+    p = chld;
+    while (xs_list_iter(&p, &v)) {
+        xs *obj = NULL;
 
-        xs *subject = xs_fmt("snac notify for @%s@%s",
-                    xs_dict_get(snac->config, "uid"), xs_dict_get(srv_config, "host"));
-        xs *from    = xs_fmt("snac-daemon <snac-daemon@%s>", xs_dict_get(srv_config, "host"));
-        xs *header  = xs_fmt(
-                    "From: %s\n"
-                    "To: %s\n"
-                    "Subject: %s\n"
-                    "\n",
-                    from, email, subject);
+        if (!valid_status(object_get_by_md5(v, &obj)))
+            continue;
 
-        xs *email_body = xs_fmt("%s%s", header, body);
+        const char *name = xs_dict_get(obj, "name");
+        const char *atto = xs_dict_get(obj, "attributedTo");
 
-        enqueue_email(email_body, 0);
+        if (name && atto) {
+            /* get the current count */
+            const xs_number *cnt = xs_dict_get(rcnt, name);
+
+            if (xs_type(cnt) == XSTYPE_NUMBER) {
+                /* if it exists, increment */
+                xs *ucnt = xs_number_new(xs_number_get(cnt) + 1);
+                rcnt = xs_dict_set(rcnt, name, ucnt);
+
+                rcpts = xs_list_append(rcpts, atto);
+            }
+        }
     }
 
-    /* telegram */
+    /* create a new list of options with their new counts */
+    xs *nopts = xs_list_new();
+    p = lopts;
+    while (xs_list_iter(&p, &v)) {
+        const xs_number *cnt = xs_dict_get(rcnt, v);
 
-    char *bot     = xs_dict_get(snac->config, "telegram_bot");
-    char *chat_id = xs_dict_get(snac->config, "telegram_chat_id");
+        if (xs_type(cnt) == XSTYPE_NUMBER) {
+            xs *d1 = xs_dict_new();
+            xs *d2 = xs_dict_new();
 
-    if (!xs_is_null(bot) && !xs_is_null(chat_id) && *bot && *chat_id)
-        enqueue_telegram(body, bot, chat_id);
+            d2 = xs_dict_append(d2, "type",       "Collection");
+            d2 = xs_dict_append(d2, "totalItems", cnt);
 
-    /* finally, store it in the notification folder */
-    if (strcmp(type, "Follow") == 0)
-        objid = xs_dict_get(msg, "id");
+            d1 = xs_dict_append(d1, "type",    "Note");
+            d1 = xs_dict_append(d1, "name",    v);
+            d1 = xs_dict_append(d1, "replies", d2);
 
-    notify_add(snac, type, utype, actor, objid);
+            nopts = xs_list_append(nopts, d1);
+        }
+    }
+
+    /* update the list */
+    msg = xs_dict_set(msg, xs_dict_get(msg, "oneOf") != NULL ? "oneOf" : "anyOf", nopts);
+
+    /* due date? */
+    int closed = 0;
+    const char *end_time = xs_dict_get(msg, "endTime");
+    if (!xs_is_null(end_time)) {
+        xs *now = xs_str_utctime(0, ISO_DATE_SPEC);
+
+        /* is now greater than the endTime? */
+        if (strcmp(now, end_time) >= 0) {
+            xs *et    = xs_dup(end_time);
+            msg       = xs_dict_set(msg, "closed", et);
+
+            closed = 1;
+        }
+    }
+
+    /* update the count of voters */
+    xs *vcnt = xs_number_new(xs_list_len(rcpts));
+    msg = xs_dict_set(msg, "votersCount", vcnt);
+
+    /* store */
+    object_add_ow(id, msg);
+
+    snac_debug(user, 1, xs_fmt("recounted poll %s", id));
+    timeline_touch(user);
+
+    /* send an update message to all voters */
+    xs *u_msg = msg_update(user, msg);
+    u_msg = xs_dict_set(u_msg, "cc", rcpts);
+
+    enqueue_message(user, u_msg);
+
+    if (closed) {
+        xs *c_msg = msg_update(user, msg);
+        notify(user, "Update", "Question", user->actor, c_msg);
+    }
+
+    return 0;
 }
 
 
@@ -1078,7 +1253,11 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
     int a_status;
     int do_notify = 0;
 
-    if (xs_is_null(actor) || xs_is_null(type)) {
+    /* question votes may not have a type */
+    if (xs_is_null(type))
+        type = "Note";
+
+    if (xs_is_null(actor)) {
         snac_debug(snac, 0, xs_fmt("malformed message"));
         return 1;
     }
@@ -1138,7 +1317,7 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
         return 1;
     }
 
-    if (strcmp(type, "Follow") == 0) {
+    if (strcmp(type, "Follow") == 0) { /** **/
         if (!follower_check(snac, actor)) {
             xs *f_msg = xs_dup(msg);
             xs *reply = msg_accept(snac, f_msg, actor);
@@ -1147,7 +1326,7 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
 
             if (xs_is_null(xs_dict_get(f_msg, "published"))) {
                 /* add a date if it doesn't include one (Mastodon) */
-                xs *date = xs_str_utctime(0, "%Y-%m-%dT%H:%M:%SZ");
+                xs *date = xs_str_utctime(0, ISO_DATE_SPEC);
                 f_msg = xs_dict_set(f_msg, "published", date);
             }
 
@@ -1162,8 +1341,8 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_log(snac, xs_fmt("repeated 'Follow' from %s", actor));
     }
     else
-    if (strcmp(type, "Undo") == 0) {
-        if (strcmp(utype, "Follow") == 0) {
+    if (strcmp(type, "Undo") == 0) { /** **/
+        if (strcmp(utype, "Follow") == 0) { /** **/
             if (valid_status(follower_del(snac, actor))) {
                 snac_log(snac, xs_fmt("no longer following us %s", actor));
                 do_notify = 1;
@@ -1175,11 +1354,11 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_debug(snac, 1, xs_fmt("ignored 'Undo' for object type '%s'", utype));
     }
     else
-    if (strcmp(type, "Create") == 0) {
+    if (strcmp(type, "Create") == 0) { /** **/
         if (is_muted(snac, actor))
             snac_log(snac, xs_fmt("ignored 'Create' + '%s' from muted actor %s", utype, actor));
 
-        if (strcmp(utype, "Note") == 0) {
+        if (strcmp(utype, "Note") == 0) { /** **/
             char *id          = xs_dict_get(object, "id");
             char *in_reply_to = xs_dict_get(object, "inReplyTo");
             xs *wrk           = NULL;
@@ -1190,9 +1369,15 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
                 snac_log(snac, xs_fmt("new 'Note' %s %s", actor, id));
                 do_notify = 1;
             }
+
+            /* if it has a "name" field, it may be a vote for a question */
+            const char *name = xs_dict_get(object, "name");
+
+            if (!xs_is_null(name) && *name && !xs_is_null(in_reply_to) && *in_reply_to)
+                update_question(snac, in_reply_to);
         }
         else
-        if (strcmp(utype, "Question") == 0) {
+        if (strcmp(utype, "Question") == 0) { /**  **/
             char *id = xs_dict_get(object, "id");
 
             if (timeline_add(snac, id, object))
@@ -1202,8 +1387,8 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_debug(snac, 1, xs_fmt("ignored 'Create' for object type '%s'", utype));
     }
     else
-    if (strcmp(type, "Accept") == 0) {
-        if (strcmp(utype, "Follow") == 0) {
+    if (strcmp(type, "Accept") == 0) { /** **/
+        if (strcmp(utype, "Follow") == 0) { /** **/
             if (following_check(snac, actor)) {
                 following_add(snac, actor, msg);
                 snac_log(snac, xs_fmt("confirmed follow from %s", actor));
@@ -1215,7 +1400,7 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_debug(snac, 1, xs_fmt("ignored 'Accept' for object type '%s'", utype));
     }
     else
-    if (strcmp(type, "Like") == 0) {
+    if (strcmp(type, "Like") == 0) { /** **/
         if (xs_type(object) == XSTYPE_DICT)
             object = xs_dict_get(object, "id");
 
@@ -1224,7 +1409,7 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
         do_notify = 1;
     }
     else
-    if (strcmp(type, "Announce") == 0) {
+    if (strcmp(type, "Announce") == 0) { /** **/
         xs *a_msg = NULL;
         xs *wrk   = NULL;
 
@@ -1255,7 +1440,7 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_log(snac, xs_fmt("error requesting 'Announce' object %s", object));
     }
     else
-    if (strcmp(type, "Update") == 0) {
+    if (strcmp(type, "Update") == 0) { /** **/
         if (strcmp(utype, "Person") == 0) {
             actor_add(actor, xs_dict_get(msg, "object"));
             timeline_touch(snac);
@@ -1263,8 +1448,8 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_log(snac, xs_fmt("updated actor %s", actor));
         }
         else
-        if (strcmp(utype, "Note") == 0) {
-            char *id = xs_dict_get(object, "id");
+        if (strcmp(utype, "Note") == 0) { /** **/
+            const char *id = xs_dict_get(object, "id");
 
             object_add_ow(id, object);
             timeline_touch(snac);
@@ -1272,19 +1457,23 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_log(snac, xs_fmt("updated post %s", id));
         }
         else
-        if (strcmp(utype, "Question") == 0) {
-            char *id = xs_dict_get(object, "id");
+        if (strcmp(utype, "Question") == 0) { /** **/
+            const char *id     = xs_dict_get(object, "id");
+            const char *closed = xs_dict_get(object, "closed");
 
             object_add_ow(id, object);
             timeline_touch(snac);
 
-            snac_log(snac, xs_fmt("updated poll %s", id));
+            snac_log(snac, xs_fmt("%s poll %s", closed == NULL ? "updated" : "closed", id));
+
+            if (closed != NULL)
+                do_notify = 1;
         }
         else
             snac_log(snac, xs_fmt("ignored 'Update' for object type '%s'", utype));
     }
     else
-    if (strcmp(type, "Delete") == 0) {
+    if (strcmp(type, "Delete") == 0) { /** **/
         if (xs_type(object) == XSTYPE_DICT)
             object = xs_dict_get(object, "id");
 
@@ -1294,11 +1483,11 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             snac_debug(snac, 1, xs_fmt("ignored 'Delete' for unknown object %s", object));
     }
     else
-    if (strcmp(type, "Pong") == 0) {
+    if (strcmp(type, "Pong") == 0) { /** **/
         snac_log(snac, xs_fmt("'Pong' received from %s", actor));
     }
     else
-    if (strcmp(type, "Ping") == 0) {
+    if (strcmp(type, "Ping") == 0) { /** **/
         snac_log(snac, xs_fmt("'Ping' requested from %s", actor));
 
         xs *rsp = msg_pong(snac, actor, xs_dict_get(msg, "id"));
@@ -1413,6 +1602,14 @@ void process_user_queue_item(snac *snac, xs_dict *q_item)
             }
         }
     }
+    else
+    if (strcmp(type, "close_question") == 0) {
+        /* the time for this question has ended */
+        const char *id = xs_dict_get(q_item, "message");
+
+        if (!xs_is_null(id))
+            update_question(snac, id);
+    }
     else
         snac_log(snac, xs_fmt("unexpected q_item type '%s'", type));
 }
