@@ -1688,7 +1688,7 @@ void notify_add(snac *snac, const char *type, const char *utype,
 {
     xs *ntid = tid(0);
     xs *fn   = xs_fmt("%s/notify/", snac->basedir);
-    xs *date = xs_str_utctime(0, "%Y-%m-%dT%H:%M:%SZ");
+    xs *date = xs_str_utctime(0, ISO_DATE_SPEC);
     FILE *f;
 
     /* create the directory */
@@ -1920,6 +1920,45 @@ void enqueue_message(snac *snac, xs_dict *msg)
 }
 
 
+void enqueue_close_question(snac *user, const char *id, int end_secs)
+/* enqueues the closing of a question */
+{
+    xs *qmsg = _new_qmsg("close_question", id, 0);
+    xs *ntid = tid(end_secs);
+    xs *fn   = xs_fmt("%s/queue/%s.json", user->basedir, ntid);
+
+    qmsg = xs_dict_set(qmsg, "ntid", ntid);
+
+    qmsg = _enqueue_put(fn, qmsg);
+
+    snac_debug(user, 0, xs_fmt("enqueue_close_question %s", id));
+}
+
+
+int was_question_voted(snac *user, const char *id)
+/* returns true if the user voted in this poll */
+{
+    xs *children = object_children(id);
+    int voted = 0;
+    xs_list *p;
+    xs_str *md5;
+
+    p = children;
+    while (xs_list_iter(&p, &md5)) {
+        xs *obj = NULL;
+
+        if (valid_status(object_get_by_md5(md5, &obj))) {
+            if (strcmp(xs_dict_get(obj, "attributedTo"), user->actor) == 0) {
+                voted = 1;
+                break;
+            }
+        }
+    }
+
+    return voted;
+}
+
+
 xs_list *user_queue(snac *snac)
 /* returns a list with filenames that can be dequeued */
 {
