@@ -117,6 +117,9 @@ xs_str *html_actor_icon(xs_str *os, char *actor,
     if (priv)
         s = xs_str_cat(s, " <span title=\"private\">&#128274;</span>");
 
+    if (strcmp(xs_dict_get(actor, "type"), "Service") == 0)
+        s = xs_str_cat(s, " <span title=\"bot\">&#129302;</span>");
+
     if (xs_is_null(date)) {
         s = xs_str_cat(s, "<br>\n&nbsp;\n");
     }
@@ -135,12 +138,37 @@ xs_str *html_actor_icon(xs_str *os, char *actor,
         }
 
         xs *s1 = xs_fmt(
-            "<br>\n<time class=\"dt-published snac-pubdate\" title=\"%s\">%s</time>\n",
+            "\n<time class=\"dt-published snac-pubdate\" title=\"%s\">%s</time>\n",
                 date_title, date_label);
 
         s = xs_str_cat(s, s1);
     }
 
+    {
+        char *username, *id;
+        xs *s1;
+
+        if (xs_is_null(username = xs_dict_get(actor, "preferredUsername")) || *username == '\0') {
+            /* This should never be reached */
+            username = "anonymous";
+        }
+
+        if (xs_is_null(id = xs_dict_get(actor, "id")) || *id == '\0') {
+            /* This should never be reached */
+            id = "https://social.example.org/anonymous";
+        }
+
+        /* "LIKE AN ANIMAL" */
+        xs *domain = xs_split(id, "/");
+        xs *user   = xs_fmt("@%s@%s", username, xs_list_get(domain, 2));
+
+        s1 = xs_fmt(
+            "<br><a href=\"%s\" class=\"p-author-tag h-card snac-author-tag\">%s</a>",
+                xs_dict_get(actor, "id"), user);
+
+        s = xs_str_cat(s, s1);
+    }
+
     return xs_str_cat(os, s);
 }
 
@@ -348,8 +376,28 @@ d_char *html_top_controls(snac *snac, d_char *s)
         "<input type=\"hidden\" name=\"in_reply_to\" value=\"\">\n"
         "<p>%s: <input type=\"checkbox\" name=\"sensitive\">\n"
         "<p>%s: <input type=\"checkbox\" name=\"mentioned_only\">\n"
+
+        "<details><summary>%s</summary>\n" /** attach **/
         "<p>%s: <input type=\"file\" name=\"attach\">\n"
         "<p>%s: <input type=\"text\" name=\"alt_text\">\n"
+        "</details>\n"
+
+        "<p>"
+        "<details><summary>%s</summary>\n" /** poll **/
+        "<p>%s:<br>\n"
+        "<textarea class=\"snac-textarea\" name=\"poll_options\" "
+        "rows=\"6\" wrap=\"virtual\"></textarea>\n"
+        "<p><select name=\"poll_multiple\">\n"
+        "<option value=\"off\">%s</option>\n"
+        "<option value=\"on\">%s</option>\n"
+        "</select>\n"
+        "<select name=\"poll_end_secs\" id=\"poll_end_secs\">\n"
+        "<option value=\"300\">%s</option>\n"
+        "<option value=\"3600\">%s</option>\n"
+        "<option value=\"86400\">%s</option>\n"
+        "</select>\n"
+        "</details>\n"
+
         "<p><input type=\"submit\" class=\"button\" value=\"%s\">\n"
         "</form><p>\n"
         "</div>\n"
@@ -357,19 +405,19 @@ d_char *html_top_controls(snac *snac, d_char *s)
         "<div class=\"snac-top-controls-more\">\n"
         "<details><summary>%s</summary>\n"
 
-        "<form method=\"post\" action=\"%s/admin/action\">\n"
+        "<form method=\"post\" action=\"%s/admin/action\">\n" /** follow **/
         "<input type=\"text\" name=\"actor\" required=\"required\">\n"
         "<input type=\"submit\" name=\"action\" value=\"%s\"> %s\n"
         "</form><p>\n"
 
-        "<form method=\"post\" action=\"%s/admin/action\">\n"
+        "<form method=\"post\" action=\"%s/admin/action\">\n" /** boost **/
         "<input type=\"text\" name=\"id\" required=\"required\">\n"
         "<input type=\"submit\" name=\"action\" value=\"%s\"> %s\n"
         "</form><p>\n"
 
         "<details><summary>%s</summary>\n"
 
-        "<div class=\"snac-user-setup\">\n"
+        "<div class=\"snac-user-setup\">\n" /** user setup **/
         "<form method=\"post\" action=\"%s/admin/user-setup\" enctype=\"multipart/form-data\">\n"
         "<p>%s:<br>\n"
         "<input type=\"text\" name=\"name\" value=\"%s\"></p>\n"
@@ -446,8 +494,19 @@ d_char *html_top_controls(snac *snac, d_char *s)
         snac->actor,
         L("Sensitive content"),
         L("Only for mentioned people"),
-        L("Image"),
-        L("Image description"),
+
+        L("Attach..."),
+        L("File"),
+        L("File description"),
+
+        L("Poll..."),
+        L("Poll options (one per line, up to 8)"),
+        L("One choice"),
+        L("Multiple choices"),
+        L("End in 5 minutes"),
+        L("End in 1 hour"),
+        L("End in 1 day"),
+
         L("Post"),
 
         L("More options..."),
@@ -617,8 +676,11 @@ xs_str *html_entry_controls(snac *snac, xs_str *os, const xs_dict *msg, const ch
 
             "<p>%s: <input type=\"checkbox\" name=\"sensitive\">\n"
             "<p>%s: <input type=\"checkbox\" name=\"mentioned_only\">\n"
+
+            "<details><summary>%s</summary>\n"
             "<p>%s: <input type=\"file\" name=\"attach\">\n"
             "<p>%s: <input type=\"text\" name=\"alt_text\">\n"
+            "</details>\n"
 
             "<input type=\"hidden\" name=\"redir\" value=\"%s_entry\">\n"
             "<p><input type=\"submit\" class=\"button\" value=\"%s\">\n"
@@ -633,8 +695,9 @@ xs_str *html_entry_controls(snac *snac, xs_str *os, const xs_dict *msg, const ch
             id,
             L("Sensitive content"),
             L("Only for mentioned people"),
-            L("Image"),
-            L("Image description"),
+            L("Attach..."),
+            L("File"),
+            L("File description"),
             md5,
             L("Post")
         );
@@ -657,8 +720,11 @@ xs_str *html_entry_controls(snac *snac, xs_str *os, const xs_dict *msg, const ch
 
             "<p>%s: <input type=\"checkbox\" name=\"sensitive\">\n"
             "<p>%s: <input type=\"checkbox\" name=\"mentioned_only\">\n"
+
+            "<details><summary>%s</summary>\n"
             "<p>%s: <input type=\"file\" name=\"attach\">\n"
             "<p>%s: <input type=\"text\" name=\"alt_text\">\n"
+            "</details>\n"
 
             "<input type=\"hidden\" name=\"redir\" value=\"%s_entry\">\n"
             "<p><input type=\"submit\" class=\"button\" value=\"%s\">\n"
@@ -673,8 +739,9 @@ xs_str *html_entry_controls(snac *snac, xs_str *os, const xs_dict *msg, const ch
             id,
             L("Sensitive content"),
             L("Only for mentioned people"),
-            L("Image"),
-            L("Image description"),
+            L("Attach..."),
+            L("File"),
+            L("File description"),
             md5,
             L("Post")
         );
@@ -734,6 +801,10 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
         return os;
     }
 
+    /* ignore notes with "name", as they are votes to Questions */
+    if (strcmp(type, "Note") == 0 && !xs_is_null(xs_dict_get(msg, "name")))
+        return os;
+
     /* bring the main actor */
     if ((actor = xs_dict_get(msg, "attributedTo")) == NULL)
         return os;
@@ -751,6 +822,12 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
         /* add the ballot box emoji */
         xs *f = xs_fmt("<span title=\"%s\"> &#128499; </span>", L("Poll"));
         s = xs_str_cat(s, f);
+
+        if (was_question_voted(snac, id)) {
+            /* add a check to show this poll was voted */
+            xs *f2 = xs_fmt("<span title=\"%s\"> &#10003; </span>", L("Voted"));
+            s = xs_str_cat(s, f2);
+        }
     }
 
     /* if this is our post, add the score */
@@ -899,7 +976,10 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
             int closed = 0;
 
             if (xs_dict_get(msg, "closed"))
-                closed = 1;
+                closed = 2;
+            else
+            if (xs_startswith(id, snac->actor))
+                closed = 1; /* we questioned; closed for us */
             else {
                 /* not yet closed? check if we already voted for this */
                 xs *children = object_children(id);
@@ -911,7 +991,7 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
                         const char *atto = xs_dict_get(msg, "attributedTo");
 
                         if (atto && strcmp(atto, snac->actor) == 0)
-                            closed = 1; /* closed for us */
+                            closed = 1; /* we already voted; closed for us */
                     }
                 }
             }
@@ -936,6 +1016,12 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
                 }
 
                 c = xs_str_cat(c, "</table>\n");
+
+                /* if it's *really* closed, say it */
+                if (closed == 2) {
+                    xs *s1 = xs_fmt("<p>%s</p>\n", L("Closed"));
+                    c = xs_str_cat(c, s1);
+                }
             }
             else {
                 /* poll still active */
@@ -976,6 +1062,10 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
 
     if ((attach = xs_dict_get(msg, "attachment")) != NULL) { /** **/
         char *v;
+
+        /* make custom css for attachments easier */
+        s = xs_str_cat(s, "<p class=\"snac-content-attachments\">\n");
+
         while (xs_list_iter(&attach, &v)) {
             char *t = xs_dict_get(v, "mediaType");
 
@@ -987,8 +1077,9 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
                 char *name = xs_dict_get(v, "name");
 
                 if (url != NULL) {
-                    xs *s1 = xs_fmt("<p><img src=\"%s\" alt=\"%s\" loading=\"lazy\"/></p>\n",
-                        url, xs_is_null(name) ? "" : name);
+                    xs *s1 = xs_fmt(
+                        "<a href=\"%s\" target=\"_blank\"><img src=\"%s\" alt=\"%s\" loading=\"lazy\"/></a>\n",
+                            url, url, xs_is_null(name) ? "" : name);
 
                     s = xs_str_cat(s, s1);
                 }
@@ -998,12 +1089,14 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
                 char *url  = xs_dict_get(v, "url");
 
                 if (url != NULL) {
-                    xs *s1 = xs_fmt("<p><object data=\"%s\"></object></p>\n", url);
+                    xs *s1 = xs_fmt("<object data=\"%s\"></object>\n", url);
 
                     s = xs_str_cat(s, s1);
                 }
             }
         }
+
+        s = xs_str_cat(s, "</p>\n");
     }
 
     if (sensitive)
@@ -1017,7 +1110,7 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
         s = html_entry_controls(snac, s, msg, md5);
 
     /** children **/
-    if (!hide_children && strcmp(type, "Question") != 0) {
+    if (!hide_children) {
         xs *children = object_children(id);
         int left     = xs_list_len(children);
 
@@ -1050,7 +1143,7 @@ xs_str *html_entry(snac *snac, xs_str *os, const xs_dict *msg, int local,
                     older_open = 0;
                 }
 
-                if (chd != NULL) {
+                if (chd != NULL && xs_is_null(xs_dict_get(chd, "name"))) {
                     ss = html_entry(snac, ss, chd, local, level + 1, cmd5, hide_children);
                     n_children++;
                 }
@@ -1274,7 +1367,7 @@ d_char *html_people(snac *snac)
 
     s = html_people_list(snac, s, wing, L("People you follow"), "i");
 
-    s = html_people_list(snac, s, wers, L("People that follows you"), "e");
+    s = html_people_list(snac, s, wers, L("People that follow you"), "e");
 
     s = html_user_footer(s);
 
@@ -1308,10 +1401,11 @@ xs_str *html_notifications(snac *snac)
             continue;
 
         xs *obj = NULL;
-        const char *type = xs_dict_get(noti, "type");
-        const char *id   = xs_dict_get(noti, "objid");
+        const char *type  = xs_dict_get(noti, "type");
+        const char *utype = xs_dict_get(noti, "utype");
+        const char *id    = xs_dict_get(noti, "objid");
 
-        if (!valid_status(object_get(id, &obj)))
+        if (xs_is_null(id) || !valid_status(object_get(id, &obj)))
             continue;
 
         if (is_hidden(snac, id))
@@ -1346,8 +1440,16 @@ xs_str *html_notifications(snac *snac)
 
         s = xs_str_cat(s, "<div>\n");
 
+        const char *label = type;
+
+        if (strcmp(type, "Create") == 0)
+            label = L("Mention");
+        else
+        if (strcmp(type, "Update") == 0 && strcmp(utype, "Question") == 0)
+            label = L("Finished poll");
+
         xs *s1 = xs_fmt("<p><b>%s by <a href=\"%s\">%s</a></b>:</p>\n",
-            strcmp(type, "Create") == 0 ? "Mention" : type, actor_id, a_name);
+            label, actor_id, a_name);
         s = xs_str_cat(s, s1);
 
         if (strcmp(type, "Follow") == 0) {
@@ -1725,8 +1827,34 @@ int html_post_handler(const xs_dict *req, const char *q_path,
             xs *msg       = NULL;
             xs *c_msg     = NULL;
             xs *content_2 = xs_replace(content, "\r", "");
+            xs *poll_opts = NULL;
 
-            msg = msg_note(&snac, content_2, to, in_reply_to, attach_list, priv);
+            /* is there a valid set of poll options? */
+            const char *v = xs_dict_get(p_vars, "poll_options");
+            if (!xs_is_null(v) && *v) {
+                xs *v2 = xs_strip_i(xs_replace(v, "\r", ""));
+
+                poll_opts = xs_split(v2, "\n");
+            }
+
+            if (!xs_is_null(poll_opts) && xs_list_len(poll_opts)) {
+                /* get the rest of poll configuration */
+                const char *p_multiple = xs_dict_get(p_vars, "poll_multiple");
+                const char *p_end_secs = xs_dict_get(p_vars, "poll_end_secs");
+                int multiple = 0;
+
+                int end_secs = atoi(!xs_is_null(p_end_secs) ? p_end_secs : "60");
+
+                if (!xs_is_null(p_multiple) && strcmp(p_multiple, "on") == 0)
+                    multiple = 1;
+
+                msg = msg_question(&snac, content_2, attach_list,
+                                   poll_opts, multiple, end_secs);
+
+                enqueue_close_question(&snac, xs_dict_get(msg, "id"), end_secs);
+            }
+            else
+                msg = msg_note(&snac, content_2, to, in_reply_to, attach_list, priv);
 
             if (sensitive != NULL) {
                 xs *t = xs_val_new(XSTYPE_TRUE);
@@ -1756,7 +1884,7 @@ int html_post_handler(const xs_dict *req, const char *q_path,
                     }
 
                     /* set the updated field */
-                    xs *updated = xs_str_utctime(0, "%Y-%m-%dT%H:%M:%SZ");
+                    xs *updated = xs_str_utctime(0, ISO_DATE_SPEC);
                     msg = xs_dict_set(msg, "updated", updated);
 
                     /* overwrite object, not updating the indexes */
