@@ -524,7 +524,7 @@ xs_dict *mastoapi_account(const xs_dict *actor)
         acct = xs_dict_append(acct, "created_at", pub);
     else {
         /* unset created_at crashes Tusky, so lie like a mf */
-        xs *date = xs_str_utctime(0, "%Y-%m-%dT%H:%M:%SZ");
+        xs *date = xs_str_utctime(0, ISO_DATE_SPEC);
         acct = xs_dict_append(acct, "created_at", date);
     }
 
@@ -587,6 +587,58 @@ xs_dict *mastoapi_account(const xs_dict *actor)
 }
 
 
+xs_dict *mastoapi_poll(snac *snac, const xs_dict *msg)
+/* creates a mastoapi Poll object */
+{
+    xs_dict *poll = xs_dict_new();
+    xs *mid       = mastoapi_id(msg);
+    xs *f         = xs_val_new(XSTYPE_FALSE);
+    xs *t         = xs_val_new(XSTYPE_TRUE);
+    xs_list *opts = NULL;
+    xs_val *v;
+    int num_votes = 0;
+    xs *options = xs_list_new();
+
+    poll = xs_dict_append(poll, "id", mid);
+    poll = xs_dict_append(poll, "expires_at", xs_dict_get(msg, "endTime"));
+    poll = xs_dict_append(poll, "expired", xs_dict_get(msg, "closed") != NULL ? t : f);
+
+    if ((opts = xs_dict_get(msg, "oneOf")) != NULL)
+        poll = xs_dict_append(poll, "multiple", f);
+    else {
+        opts = xs_dict_get(msg, "anyOf");
+        poll = xs_dict_append(poll, "multiple", t);
+    }
+
+    while (xs_list_iter(&opts, &v)) {
+        const char *title   = xs_dict_get(v, "name");
+        const char *replies = xs_dict_get(v, "replies");
+
+        if (title && replies) {
+            const char *votes_count = xs_dict_get(replies, "totalItems");
+
+            if (xs_type(votes_count) == XSTYPE_NUMBER) {
+                xs *d = xs_dict_new();
+                d = xs_dict_append(d, "title",  title);
+                d = xs_dict_append(d, "votes_count", votes_count);
+
+                options = xs_list_append(options, d);
+                num_votes += xs_number_get(votes_count);
+            }
+        }
+    }
+
+    poll = xs_dict_append(poll, "options", options);
+    xs *vc = xs_number_new(num_votes);
+    poll = xs_dict_append(poll, "votes_count", vc);
+
+    poll = xs_dict_append(poll, "voted",
+            was_question_voted(snac, xs_dict_get(msg, "id")) ? t : f);
+
+    return poll;
+}
+
+
 xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
 /* converts an ActivityPub note to a Mastodon status */
 {
@@ -597,6 +649,9 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
     if (actor == NULL)
         return NULL;
 
+    const char *type = xs_dict_get(msg, "type");
+    const char *id   = xs_dict_get(msg, "id");
+
     xs *acct = mastoapi_account(actor);
 
     xs *f   = xs_val_new(XSTYPE_FALSE);
@@ -604,9 +659,7 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
     xs *n   = xs_val_new(XSTYPE_NULL);
     xs *idx = NULL;
     xs *ixc = NULL;
-
     char *tmp;
-    char *id = xs_dict_get(msg, "id");
     xs *mid  = mastoapi_id(msg);
 
     xs_dict *st = xs_dict_new();
@@ -803,6 +856,11 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
 
     st = xs_dict_append(st, "edited_at", tmp);
 
+    if (strcmp(type, "Question") == 0) {
+        xs *poll = mastoapi_poll(snac, msg);
+        st = xs_dict_append(st, "poll", poll);
+    }
+
     return st;
 }
 
@@ -1148,7 +1206,8 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
                     continue;
 
                 /* discard non-Notes */
-                if (strcmp(xs_dict_get(msg, "type"), "Note") != 0)
+                const char *type = xs_dict_get(msg, "type");
+                if (strcmp(type, "Note") != 0 && strcmp(type, "Question") != 0)
                     continue;
 
                 /* discard notes from muted morons */
@@ -1214,7 +1273,8 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
                 continue;
 
             /* discard non-Notes */
-            if (strcmp(xs_dict_get(msg, "type"), "Note") != 0)
+            const char *type = xs_dict_get(msg, "type");
+            if (strcmp(type, "Note") != 0 && strcmp(type, "Question") != 0)
                 continue;
 
             /* convert the Note into a Mastodon status */
@@ -1253,6 +1313,7 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
                     continue;
 
                 const char *type  = xs_dict_get(noti, "type");
+                const char *utype = xs_dict_get(noti, "utype");
                 const char *objid = xs_dict_get(noti, "objid");
                 xs *actor = NULL;
                 xs *entry = NULL;
@@ -1278,6 +1339,9 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
                 else
                 if (strcmp(type, "Create") == 0)
                     type = "mention";
+                else
+                if (strcmp(type, "Update") == 0 && strcmp(utype, "Question") == 0)
+                    type = "poll";
                 else
                     continue;
 
@@ -1500,8 +1564,10 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
                         xs *m2 = NULL;
 
                         if (valid_status(timeline_get_by_md5(&snac1, v, &m2))) {
-                            xs *st = mastoapi_status(&snac1, m2);
-                            des = xs_list_append(des, st);
+                            if (xs_is_null(xs_dict_get(m2, "name"))) {
+                                xs *st = mastoapi_status(&snac1, m2);
+                                des = xs_list_append(des, st);
+                            }
                         }
                     }
 
@@ -2051,6 +2117,69 @@ int mastoapi_post_handler(const xs_dict *req, const char *q_path,
         else
             status = 401;
     }
+    else
+    if (xs_startswith(cmd, "/v1/polls")) { /** **/
+        if (logged_in) {
+            /* operations on a status */
+            xs *l = xs_split(cmd, "/");
+            const char *mid = xs_list_get(l, 3);
+            const char *op  = xs_list_get(l, 4);
+
+            if (!xs_is_null(mid)) {
+                xs *msg = NULL;
+                xs *out = NULL;
+
+                /* skip the 'fake' part of the id */
+                mid = MID_TO_MD5(mid);
+
+                if (valid_status(timeline_get_by_md5(&snac, mid, &msg))) {
+                    const char *id   = xs_dict_get(msg, "id");
+                    const char *atto = xs_dict_get(msg, "attributedTo");
+
+                    xs_list *opts = xs_dict_get(msg, "oneOf");
+                    if (opts == NULL)
+                        opts = xs_dict_get(msg, "anyOf");
+
+                    if (op == NULL) {
+                    }
+                    else
+                    if (strcmp(op, "votes") == 0) {
+                        xs_list *choices = xs_dict_get(args, "choices[]");
+
+                        if (xs_type(choices) == XSTYPE_LIST) {
+                            xs_str *v;
+
+                            while (xs_list_iter(&choices, &v)) {
+                                int io           = atoi(v);
+                                const xs_dict *o = xs_list_get(opts, io);
+
+                                if (o) {
+                                    const char *name = xs_dict_get(o, "name");
+
+                                    xs *msg = msg_note(&snac, "", atto, (char *)id, NULL, 1);
+                                    msg = xs_dict_append(msg, "name", name);
+
+                                    xs *c_msg = msg_create(&snac, msg);
+                                    enqueue_message(&snac, c_msg);
+                                    timeline_add(&snac, xs_dict_get(msg, "id"), msg);
+                                }
+                            }
+
+                            out = mastoapi_poll(&snac, msg);
+                        }
+                    }
+                }
+
+                if (out != NULL) {
+                    *body  = xs_json_dumps_pp(out, 4);
+                    *ctype = "application/json";
+                    status = 200;
+                }
+            }
+        }
+        else
+            status = 401;
+    }
 
     /* user cleanup */
     if (logged_in)
