@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2023 grunfink / MIT license */
 
-#define VERSION "2.33"
+#define VERSION "2.34"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -9,6 +9,8 @@
 
 #define DIR_PERM 02770
 
+#define ISO_DATE_SPEC "%Y-%m-%dT%H:%M:%SZ"
+
 extern double disk_layout;
 extern d_char *srv_basedir;
 extern d_char *srv_config;
@@ -163,6 +165,8 @@ void enqueue_output_by_actor(snac *snac, xs_dict *msg, const xs_str *actor, int
 void enqueue_email(xs_str *msg, int retries);
 void enqueue_telegram(const xs_str *msg, const char *bot, const char *chat_id);
 void enqueue_message(snac *snac, char *msg);
+void enqueue_close_question(snac *user, const char *id, int end_secs);
+int was_question_voted(snac *user, const char *id);
 
 xs_list *user_queue(snac *snac);
 xs_list *queue(void);
@@ -205,7 +209,8 @@ d_char *msg_actor(snac *snac);
 xs_dict *msg_update(snac *snac, xs_dict *object);
 xs_dict *msg_ping(snac *user, const char *rcpt);
 xs_dict *msg_pong(snac *user, const char *rcpt, const char *object);
-xs_dict *msg_question(snac *user, const char *content, const xs_list *opts, int multiple, int end_secs);
+xs_dict *msg_question(snac *user, const char *content, xs_list *attach,
+                      const xs_list *opts, int multiple, int end_secs);
 
 int activitypub_request(snac *snac, const char *url, xs_dict **data);
 int actor_request(snac *snac, const char *actor, xs_dict **data);
