@@ -38,7 +38,8 @@ const char *default_css =
     ".snac-top-user-name { font-size: 200% }\n"
     ".snac-top-user-id { font-size: 150% }\n"
     ".snac-avatar { float: left; height: 2.5em; padding: 0.25em }\n"
-    ".snac-author { font-size: 90% }\n"
+    ".snac-author { font-size: 90%; text-decoration: none }\n"
+    ".snac-author-tag { font-size: 80% }\n"
     ".snac-pubdate { color: #a0a0a0; font-size: 90% }\n"
     ".snac-top-controls { padding-bottom: 1.5em }\n"
     ".snac-post { border-top: 1px solid #a0a0a0; }\n"
@@ -218,7 +219,7 @@ int adduser(const char *uid)
 {
     snac snac;
     xs *config = xs_dict_new();
-    xs *date = xs_str_utctime(0, "%Y-%m-%dT%H:%M:%SZ");
+    xs *date = xs_str_utctime(0, ISO_DATE_SPEC);
     xs *pwd = NULL;
     xs *pwd_f = NULL;
     xs *key = NULL;
