@@ -1,4 +1,4 @@
-/* copyright (c) 2022 grunfink - MIT license */
+/* copyright (c) 2022 - 2023 grunfink / MIT license */
 
 #ifndef _XS_MIME
 
@@ -23,6 +23,8 @@ struct _mime_info {
     { "text/html",          ".html" },
     { "text/plain",         ".txt" },
     { "text/xml",           ".xml" },
+    { "text/markdown",      ".md" },
+    { "text/gemini",        ".gmi" },
     { NULL, NULL }
 };
 
