@@ -1,4 +1,4 @@
-/* copyright (c) 2022 grunfink - MIT license */
+/* copyright (c) 2022 - 2023 grunfink / MIT license */
 
 #ifndef _XS_TIME_H
 
@@ -6,17 +6,18 @@
 
 #include <time.h>
 
-d_char *xs_str_time(time_t t, const char *fmt, int local);
+xs_str *xs_str_time(time_t t, const char *fmt, int local);
 #define xs_str_localtime(t, fmt) xs_str_time(t, fmt, 1)
 #define xs_str_utctime(t, fmt)   xs_str_time(t, fmt, 0)
 time_t xs_parse_time(const char *str, const char *fmt, int local);
 #define xs_parse_localtime(str, fmt) xs_parse_time(str, fmt, 1)
 #define xs_parse_utctime(str, fmt) xs_parse_time(str, fmt, 0)
+xs_str *xs_str_time_diff(time_t time_diff);
 
 #ifdef XS_IMPLEMENTATION
 
-d_char *xs_str_time(time_t t, const char *fmt, int local)
-/* returns a d_char with a formated time */
+xs_str *xs_str_time(time_t t, const char *fmt, int local)
+/* returns a string with a formated time */
 {
     struct tm tm;
     char tmp[64];
@@ -37,6 +38,18 @@ d_char *xs_str_time(time_t t, const char *fmt, int local)
 }
 
 
+xs_str *xs_str_time_diff(time_t time_diff)
+/* returns time_diff in seconds to 'human' units (d:hh:mm:ss) */
+{
+    int secs  = time_diff % 60;
+    int mins  = (time_diff /= 60) % 60;
+    int hours = (time_diff /= 60) % 24;
+    int days  = (time_diff /= 24);
+
+    return xs_fmt("%d:%02d:%02d:%02d", days, hours, mins, secs);
+}
+
+
 char *strptime(const char *s, const char *format, struct tm *tm);
 
 time_t xs_parse_time(const char *str, const char *fmt, int local)
